# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>

   # include "color_theme.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "header_line.h"
   # include "message.h"
   # include "mycurses.h"
   # include "preferences.h"
   # include "status_line.h"
   # include "screen.h"
# endif
# if GLOBAL
      // -- color pairs --
   short header_color_pair = 1;
   short status_color_pair = 2;
   short screen_color_pair = 3;
   short block_color_pair = 4;
   short selection_color_pair = 5;
   short search_color_pair = 6;
   short browse_color_pair = 7;

   Color_Theme * _user_color_theme;
   static int color_theme_data[] = 
      {    // -- color theme 0 "Standard" --
          0,   0,   0, //0 black
        680,   0,   0, //1 red
          0, 680,   0, //2 green
        680, 680,   0, //3 yellow
          0,   0, 680, //4 blue
        680,   0, 680, //5 magenta
          0, 680, 680, //6 cyan
        680, 680, 680, //white
          7, 4, 0, 6, 7, 0, 3, 4, 7, 5, 0, 7,
           // -- color theme 1 "New" --
        300, 300, 300,
        500, 500, 500,
        310,  20, 350,
        310, 300, 410,
        500, 500, 500,
        500, 500, 500,
        380,  30, 290,
        500, 500, 500,
          1, 2, 3, 4, 7, 0, 3, 4, 5, 6, 0, 7,
           // -- color theme 2 "Midnight" --
          0,   0,   0,
          0,   0,   0,
        650,   0,   0,
          0,   0,   0,
        630, 530,   0,
          0,   0,   0,
        650,   0,   0,
          0, 680,   0,
          0, 2, 3, 4, 7, 0, 1, 4, 5, 6, 0, 7,
            // -- color theme 3 "Blue" --
           0, 100, 410,
           0, 100, 410,
         220, 520, 690,
         330, 580, 670,
           0, 140, 610,
         600, 510,   0,
         530,   0, 290,
         390, 470, 600,
           1, 2, 3, 4, 7, 0, 3, 4, 5, 6, 0, 7,
      };
   int color_theme_cardinal = sizeof(color_theme_data)/sizeof(Color_Theme);
   Color_Theme * _color_theme[sizeof(color_theme_data)/sizeof(Color_Theme)];
# endif
# if PROTO
   void color_pair_init(void);
# endif

void color_theme_init()
   { //printf("%d\n", sizeof(color_theme_data)/sizeof(Color_Theme));
     //endwin();
     //exit(0);
     for (int i = 0; i < sizeof(color_theme_data)/sizeof(Color_Theme); i++)
        { _color_theme[i] = (Color_Theme *) (color_theme_data + i*sizeof(Color_Theme)/sizeof(int));
        }
     return;
   }
void color_theme_set(int ordinal)
   { _user_color_theme = _color_theme[ordinal];
     _preferences->color_theme = ordinal;
     int id;
     Color * _color_new;
     for (int i = 0; i < 8; i++)
        { //id = _user_color_theme->color[i];
          //printf("%d %d %d\n", _color[id]->red, _color[id]->green, _color[id]->blue);
          _color_new = & _user_color_theme->color[i];
          //curses_init_color(i, _color[id]->red, _color[id]->green, _color[id]->blue);
          curses_init_color(i, _color_new->red, _color_new->green, _color_new->blue);
        }
     color_pair_init();
     curses_touchwin(stdscr);
     return;
   }
void color_pair_init()
   { curses_init_pair(header_color_pair, _user_color_theme->header_line.foreground, _user_color_theme->header_line.background);
     curses_init_pair(status_color_pair, _user_color_theme->status_line.foreground, _user_color_theme->status_line.background);
     curses_init_pair(screen_color_pair, _user_color_theme->screen.foreground, _user_color_theme->screen.background);
     curses_init_pair(block_color_pair, _user_color_theme->block.foreground, _user_color_theme->block.background);
     curses_init_pair(search_color_pair, _user_color_theme->search.foreground, _user_color_theme->search.background);
     curses_init_pair(selection_color_pair, _user_color_theme->selection.foreground, _user_color_theme->selection.background);
     return;
   }

void colors_browse()
   { screen_clear();
     status_line_print(_message[59]);
     cursor_hide();
     mvprintw(4+0, 3, _message[60]);
     mvprintw(4+1, 3, _message[61]);
     mvprintw(4+2, 3, _message[62]);
     mvprintw(4+3, 3, _message[63]);
     mvprintw(4+4, 3, _message[64]);
     mvprintw(4+5, 3, _message[65]);
     int color_browse = 6;
     curses_init_pair(browse_color_pair, 0, color_browse);
     short color_r, color_g, color_b;
     short * _color_cur = & color_r;
     curses_color_content(color_browse, & color_r, & color_g, & color_b);
     int box_s = 6;
     int key = 'r';
     int key_color = 'r';
     int text_line = 4;
     int base_r = 30;
     int step = 10;
     do
        { //mvprintw(4, 27, "%4d %4d %4d", color_r, color_g, color_b);
          switch ( key )
            { case KEY_UP:
                 if ( * _color_cur + step < 1000 )
                    { * _color_cur += step;
                    }
                 break;
              case KEY_DOWN:
                 if ( * _color_cur - step >= 0 )
                    { * _color_cur -= step;
                    }
                 break;
              case 'i':
                 step++;
                 break;
              case 'd':
                 step--;
                 break;
              case '1':
                 color_browse = 1;
                 break;
              case '2':
                 color_browse = 2;
                 break;
              case '3':
                 color_browse = 3;
                 break;
              case '4':
                 color_browse = 4;
                 break;
              case '5':
                 color_browse = 5;
                 break;
              case '6':
                 color_browse = 6;
                 break;
              case '7':
                 color_browse = 7;
                 break;
              case '0':
                 color_browse = 0;
                 break;
            }
          if ( (key >= '1' && key <= '9') || key == '0' )
             { curses_color_content(color_browse, & color_r, & color_g, & color_b);
             }
          curses_init_color(color_browse, color_r, color_g, color_b);
          curses_init_pair(browse_color_pair, 0, color_browse);
          for (int i = 0; i < box_s; i++)
             { curses_mvchgat(10+i, 26, 16, A_NORMAL, browse_color_pair, 0);
             }
          curses_touchline(stdscr, 10, 20);
          mvprintw(text_line+0, base_r, "%s  %3d", _message[67], color_r);
          mvprintw(text_line+1, base_r, "%s  %3d", _message[68], color_g);
          mvprintw(text_line+2, base_r, "%s  %3d", _message[69], color_b);
          mvprintw(text_line+3, base_r, "%s  %3d", _message[70], step);
          mvprintw(text_line+4, base_r, "%s  %3d", _message[71], color_browse);
          switch( key_color )
             { case 'r':
                  curses_mvchgat(text_line+0, base_r+8, 3, A_REVERSE, screen_color_pair, 0);
                  curses_touchline(stdscr, text_line, 1);
                  _color_cur = & color_r;
                  break;
               case 'g':
                  curses_mvchgat(text_line+1, base_r+8, 3, A_REVERSE, screen_color_pair, 0);
                  curses_touchline(stdscr, text_line+1, 1);
                  _color_cur = & color_g;
                  break;
               case 'b':
                  curses_mvchgat(text_line+2, base_r+8, 3, A_REVERSE, screen_color_pair, 0);
                  curses_touchline(stdscr, text_line+2, 1);
                  _color_cur = & color_b;
                  break;
             }
          key = curses_getch();
          if ( key == 'r' || key == 'g' || key == 'b' )
              { key_color = key;
              }
        }
     while ( key != 'q');

     header_line_refresh();
     status_line_refresh();
     screen_redraw();
     cursor_show();
     return;
   }
