# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>
   # include <time.h>

   # include "cwrite.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "mycurses.h"
   # include "screen.h"
   # include "scroll.h"
   # include "string.h"
   # include "status_line.h"
# endif
# if GLOBAL
   int * _cursor_y;
   int * _cursor_x;
# endif
# if PROTO
   int min(int x, int y);
# endif

void cursor_show()
   { curses_curs_set(1);
     return;
   }
void cursor_hide()
   { curses_curs_set(0);
     return;
   }

void cursor_move(int y, int x)
   { curses_move(y, x);
     return;
   }

bool cursor_left()
   { if ( ! * _dataline_offset )
        { return 0;
        }
     (* _dataline_offset)--;
     if ( * _cursor_x == screen_left )
        { screen_redraw();
        }
     else
        { (* _cursor_x)--;
        }
     cursor_move(* _cursor_y, * _cursor_x);
     return 1;
   } 
bool cursor_right()
   { if ( * _dataline_offset == _datasystem->dataline_max_size )
        { return 0;
        }
     (* _dataline_offset)++;

     if ( * _cursor_x == screen_right )
        { screen_redraw();
        }
     else
        { (* _cursor_x)++;
        }
     cursor_move(* _cursor_y, * _cursor_x);
     return 1;
    
   }

bool cursor_up()
   { if ( ! seek_visible_line_up() )
        { return 0;
        }
     if ( * _cursor_y == screen_top )
        { screen_redraw();
        }
     else
        { (* _cursor_y)--;
        }
     cursor_move(* _cursor_y, * _cursor_x);
     return 1;
   }
bool cursor_down()
   { if ( ! seek_visible_line_down() )
        { return 0;
        }
     if ( * _cursor_y == screen_bottom )
        { screen_redraw();
        }
     else
        { (* _cursor_y)++;
        }
     cursor_move(* _cursor_y, * _cursor_x);
     return 1;
   }

void cursor_home()
   { if ( * _cursor_x == screen_left )
        { return;
        }
     * _cursor_x = screen_left;
     cursor_move(* _cursor_y, * _cursor_x);
     * _dataline_offset = 0;
     screen_redraw();
     return;
   }
void cursor_end()
   { int size = (* __datachain)->size;
     if ( * _dataline_offset == size )
        { * _dataline_offset = _datasystem->dataline_max_size;
          * _cursor_x = screen_right;
        }
     else
        { * _dataline_offset = size;
          if ( screen_left + size < screen_right )
             { * _cursor_x = screen_left + size;
             }
          else
             { * _cursor_x = screen_right;
             }
        }
     cursor_move(* _cursor_y, * _cursor_x);
     screen_redraw();
     return; 
   }

void cursor_npage()
   { if ( * _cursor_y < screen_bottom )
        { int n = screen_bottom;
          int visible_lines = visible_lines_down(* __datachain);
          if ( visible_lines < screen_bottom )
             { n = min(visible_lines + * _cursor_y, screen_bottom);
             }
          int cursor_y = * _cursor_y;
          for ( int i = cursor_y; i < n; i++ )
             { cursor_down();
             }
        }
     else
        { scroll_npage();
        }
     return;
   }
void cursor_ppage()
   { if ( * _cursor_y > screen_top )
        { while ( * _cursor_y > screen_top )
            { cursor_up();
            }
        }
     else
        { scroll_ppage();
        }
     return;
   }

void cursor_delete()
   { if ( ! (* _status_register & TYPE_MODE) )
        { return;
        }
     if ( * _dataline_offset >= (* __datachain)->size )
        { return;
        }
     dataline_shift_left(* __datachain, 1);
     screen_line_redraw(* __datachain, * _cursor_y);
     return;
   }
void cursor_backspace()
   { if ( ! (* _status_register & TYPE_MODE) )
        { return;
        }
     if ( ! cursor_left() )
        { return;
        }
     cursor_delete();
     return;
   }

void show_coords()
   { status_line_vprintf((char *) "(%d, %d)", (* __datachain)->ordinal+1, * _dataline_offset + 1);
     return;
   }
void show_date()
   { time_t current_time = time(0);
     struct tm * _local_time = localtime(& current_time);
     strftime(_status_line_message, 22, "%a %b %d %Y", _local_time);
     status_line_redraw();
     status_line_busy = 1;
     return;
   }

int min(int x, int y)
   { return x < y ? x : y;
   }
